@@ -54,6 +54,7 @@ enum
 {
 	DIR_DOUBLE_CLICKED_SIGNAL,
 	UP_DIR_REQUEST_SIGNAL,
+	SEARCH_END_SIGNAL,
 	LAST_SIGNAL
 };
 
@@ -101,6 +102,10 @@ static void cxp_right_pane_draw_filename (GtkTreeViewColumn * tree_column,
 					  GtkCellRenderer * cell,
 					  GtkTreeModel * tree_model,
 					  GtkTreeIter * iter, gpointer data);
+static void cxp_right_pane_draw_place (GtkTreeViewColumn * tree_column,
+					  GtkCellRenderer * cell,
+					  GtkTreeModel * tree_model,
+					  GtkTreeIter * iter, gpointer data);
 static void cxp_right_pane_draw_filetype (GtkTreeViewColumn * tree_column,
 					  GtkCellRenderer * cell,
 					  GtkTreeModel * tree_model,
@@ -142,6 +147,8 @@ static void cxp_right_pane_drag_data_delete (GtkWidget *widget, GdkDragContext *
 static void cxp_right_pane_drag_data_received (GtkWidget *widget, GdkDragContext *dc, gint x, gint y, GtkSelectionData *selection_data, guint info, guint t, gpointer data);
 static gboolean cxp_right_pane_drag_drop (GtkWidget *widget, GdkDragContext *dc, gint x, gint y, guint t, gpointer data);
 static gchar *cxp_right_pane_get_drag_dest_dir (CxpRightPane *obj, gint x, gint y);
+static gboolean cxp_right_pane_file_found (GIOChannel *source, GIOCondition condition, gpointer data);
+static void cxp_right_pane_child_watch (GPid pid, gint status, gpointer data);
 
 GType cxp_right_pane_get_type (void)
 {
@@ -206,6 +213,15 @@ static void cxp_right_pane_class_init (CxpRightPaneClass * klass)
 			      NULL, g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
+	cxp_right_pane_signals[SEARCH_END_SIGNAL] =
+		g_signal_new ("search_end",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (CxpRightPaneClass,
+					       search_end), NULL,
+			      NULL, g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+
 
 	parent_class = g_type_class_peek_parent (klass);
 }
@@ -221,7 +237,7 @@ static void cxp_right_pane_instance_init (CxpRightPane * self)
 	//gtk_paned_set_position (GTK_PANED (self), 250);
 
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_set_size_request (GTK_WIDGET (scrolled_window), 300, 200);
+	gtk_widget_set_size_request (GTK_WIDGET (scrolled_window), 400, 300);
 	gtk_box_pack_start (GTK_BOX (self), scrolled_window, TRUE, TRUE, 0);
 	gtk_widget_show (scrolled_window);
 	priv->file_list = cxp_right_pane_file_list_new (self);
@@ -351,6 +367,7 @@ static GtkWidget *cxp_right_pane_file_list_new (CxpRightPane * self)
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
 	GtkCellRenderer *renderer;
+
 	GtkTreeSortable *sortable;
 
 	file_list = gtk_tree_view_new ();
@@ -374,6 +391,8 @@ static GtkWidget *cxp_right_pane_file_list_new (CxpRightPane * self)
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_title (column, _("Filename"));
 	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_fixed_width (column, 150);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (file_list), column);
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
@@ -393,6 +412,20 @@ static GtkWidget *cxp_right_pane_file_list_new (CxpRightPane * self)
 			  G_CALLBACK (cxp_right_pane_cell_edited), self);
 
 	/* 2nd column */
+	column = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_title (column, _("Place"));
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_fixed_width (column, 150);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (file_list), column);
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_tree_view_column_pack_start (column, renderer, FALSE);
+	gtk_tree_view_column_set_cell_data_func (column, renderer,
+						 cxp_right_pane_draw_place,
+						 self, NULL);
+	gtk_tree_view_column_set_visible (column, FALSE);
+
+	/* 3rd column */
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer, "xalign", 1.0, NULL);
 	column = gtk_tree_view_column_new_with_attributes (_("Size"), renderer,
@@ -403,7 +436,7 @@ static GtkWidget *cxp_right_pane_file_list_new (CxpRightPane * self)
 	gtk_tree_view_column_set_sort_column_id (column, SORT_ID_SIZE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (file_list), column);
 
-	/* 3rd column */
+	/* 4th column */
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Type"), renderer,
 							   NULL);
@@ -416,7 +449,7 @@ static GtkWidget *cxp_right_pane_file_list_new (CxpRightPane * self)
 	gtk_tree_view_column_set_sort_column_id (column, SORT_ID_TYPE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (file_list), column);
 
-	/* 4th column */
+	/* 5th column */
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Update Time"),
 							   renderer, NULL);
@@ -611,6 +644,26 @@ static void cxp_right_pane_draw_filename (GtkTreeViewColumn * tree_column,
 	g_free (fullpath);
 }
 
+static void cxp_right_pane_draw_place (GtkTreeViewColumn * tree_column,
+					  GtkCellRenderer * cell,
+					  GtkTreeModel * tree_model,
+					  GtkTreeIter * iter, gpointer data)
+{
+	gchar *fullpath;
+	gchar *place;
+
+	gtk_tree_model_get (tree_model, iter, FILE_LIST_COL_FULLPATH, &fullpath,
+			    -1);
+	if ((place = cxp_path_get_dirname_of_utf8 (fullpath)) == NULL)
+	{
+		place = g_strdup (_("(Invalid filename)"));
+	}
+
+	g_object_set (cell, "text", place, NULL);
+
+	g_free (fullpath);
+}
+
 static void cxp_right_pane_draw_filetype (GtkTreeViewColumn * tree_column,
 					  GtkCellRenderer * cell,
 					  GtkTreeModel * tree_model,
@@ -1074,10 +1127,13 @@ void cxp_right_pane_change_directory (CxpRightPane * self,
 	gchar *fullpath;
 	GDir *dir;
 	GtkTreeView *file_list;
+	GtkTreeViewColumn *column;
 	GtkTreeModel *model;
 	GtkTextBuffer *text_buffer;
 	gboolean show_dot_file;
 
+	column = gtk_tree_view_get_column (GTK_TREE_VIEW(priv->file_list), 1);
+	gtk_tree_view_column_set_visible (column, FALSE);
 	cur_dir = g_strdup (dirname);
 	show_dot_file =
 		gconf_client_get_bool (priv->client,
@@ -1652,3 +1708,70 @@ static gchar *cxp_right_pane_get_drag_dest_dir (CxpRightPane *obj, gint x, gint
 
 	return	result;
 }
+
+static gboolean cxp_right_pane_file_found (GIOChannel *source, GIOCondition condition, gpointer data)
+{
+	CxpRightPanePrivate *priv = CXP_RIGHT_PANE_GET_PRIVATE (data);
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gchar *line;
+	
+	if (g_io_channel_read_line (source, &line, NULL, NULL, NULL) == G_IO_STATUS_NORMAL)
+	{
+		g_strchomp (line);
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW(priv->file_list));
+		cxp_file_list_store_append_file (CXP_FILE_LIST_STORE(model), line, &iter);
+		if (GTK_WIDGET_VISIBLE(priv->file_list) && GTK_WIDGET_DRAWABLE(priv->file_list))
+			gdk_window_process_updates(priv->file_list->window, FALSE);
+		g_free (line);
+	}
+
+	return	TRUE;
+}
+
+static void cxp_right_pane_child_watch (GPid pid, gint status, gpointer data)
+{
+	g_spawn_close_pid (pid);
+	g_signal_emit (data, cxp_right_pane_signals[SEARCH_END_SIGNAL], 0);
+}
+
+void cxp_right_pane_search_file (CxpRightPane *right_pane, const gchar *cmd)
+{
+	CxpRightPanePrivate *priv = CXP_RIGHT_PANE_GET_PRIVATE (right_pane);
+	GIOChannel *channel;
+	GPid pid;
+	GtkTreeModel *model;
+	GtkTreeViewColumn *column;
+	gchar **args;
+	gint sout;
+	GError *error = NULL;
+
+	column = gtk_tree_view_get_column (GTK_TREE_VIEW(priv->file_list), 1);
+	gtk_tree_view_column_set_visible (column, TRUE);
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW(priv->file_list));
+	g_object_ref (model);
+	gtk_tree_view_set_model (GTK_TREE_VIEW(priv->file_list), NULL);
+	cxp_file_list_store_clear (CXP_FILE_LIST_STORE (model));
+	gtk_tree_view_set_model (GTK_TREE_VIEW(priv->file_list), model);
+	g_object_unref (model);
+
+	if (g_shell_parse_argv (cmd, NULL, &args, NULL))
+	{
+		if (g_spawn_async_with_pipes (NULL, args, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL, &sout, NULL, &error))
+		{
+			g_child_watch_add_full (G_PRIORITY_DEFAULT, pid, cxp_right_pane_child_watch, right_pane, NULL);
+			channel = g_io_channel_unix_new (sout);
+			g_io_channel_set_encoding (channel, NULL, NULL);
+			g_io_add_watch (channel, G_IO_IN, cxp_right_pane_file_found, right_pane);
+		}
+		else
+		{
+			g_warning ("%s\n", error->message);
+		}
+	}
+	else
+	{
+			g_warning ("%s\n", error->message);
+	}
+}
