@@ -64,6 +64,8 @@ typedef struct
 	GtkUIManager *ui_manager;
 	GtkWidget *menubar;
 	GtkWidget *toolbar;
+	GtkWidget *scombo;
+	GtkWidget *sentry;
 	GHashTable *dynamic_widgets;
 	GtkWidget *entry;
 	GtkWidget *dirview;
@@ -91,6 +93,8 @@ static GSList* cxplorer_window_menu_item_new_from_bookmark (const gchar *key, co
 static void cxplorer_window_dispose (GObject * obj);
 static void cxplorer_window_finalize (GObject * obj);
 static void cxplorer_window_entry_activate (GtkWidget *widget, gpointer user_data);
+static void cxplorer_window_search_entry_activate  (GtkWidget *widget, gpointer user_data);
+static void cxplorer_window_search_end (GtkWidget *widget, gpointer user_data);
 static void cxplorer_window_on_directory_changed (CxpDirView * dirview,
 						  gpointer user_data);
 static void cxplorer_window_on_dir_double_clicked (CxpRightPane * right_pane,
@@ -246,9 +250,24 @@ static void cxplorer_window_instance_init (GTypeInstance * instance,
 	/* ǥ֥åޡ˥塼ۤ롣 */
 	cxplorer_window_bookmark_menu_init (self);
 
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 	toolbar = gtk_ui_manager_get_widget (private->ui_manager, "/toolbar");
-	gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), toolbar, TRUE, TRUE, 0);
 	gtk_widget_show (toolbar);
+	private->scombo = gtk_combo_box_new_text ();
+	gtk_box_pack_start (GTK_BOX (hbox), private->scombo, FALSE, FALSE, 0);
+	gtk_combo_box_append_text (GTK_COMBO_BOX(private->scombo), "find");
+	gtk_combo_box_append_text (GTK_COMBO_BOX(private->scombo), "locate");
+	/*
+	gtk_combo_box_append_text (GTK_COMBO_BOX(private->scombo), "namazu");
+	*/
+	gtk_combo_box_set_active (GTK_COMBO_BOX(private->scombo), 0);
+	gtk_widget_show (private->scombo);
+	private->sentry = gtk_entry_new ();
+	gtk_box_pack_start (GTK_BOX (hbox), private->sentry, FALSE, FALSE, 0);
+	gtk_widget_show (private->sentry);
+	gtk_widget_show (hbox);
 
 	hbox = gtk_hbox_new (FALSE, 5);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 5);
@@ -301,6 +320,9 @@ static void cxplorer_window_instance_init (GTypeInstance * instance,
 	g_signal_connect (entry, "activate",
 			  G_CALLBACK (cxplorer_window_entry_activate),
 			  self);
+	g_signal_connect (private->sentry, "activate",
+			  G_CALLBACK (cxplorer_window_search_entry_activate),
+			  self);
 	g_signal_connect (dirview, "directory_changed",
 			  G_CALLBACK (cxplorer_window_on_directory_changed),
 			  self);
@@ -310,6 +332,9 @@ static void cxplorer_window_instance_init (GTypeInstance * instance,
 	g_signal_connect (right_pane, "up_dir_request",
 			  G_CALLBACK (cxplorer_window_go_up_action),
 			  self);
+	g_signal_connect (right_pane, "search_end",
+			  G_CALLBACK (cxplorer_window_search_end),
+			  self);
 	g_signal_connect (private->client, "value_changed", G_CALLBACK(cxplorer_window_gconf_value_changed), self);
 }
 
@@ -533,6 +558,57 @@ GtkWidget *cxplorer_window_new (void)
 	return window;
 }
 
+static void cxplorer_window_search_entry_activate  (GtkWidget *widget, gpointer user_data)
+{
+	CxplorerWindowPrivate *private = CXPLORER_WINDOW_GET_PRIVATE (user_data);
+	const gchar *pattern;
+	gchar *pat_locale;
+	gchar *cmd;
+	gchar *cmd_utf8;
+	gchar *cur_dir;
+	gint index;
+	gint cid;
+
+	pattern = gtk_entry_get_text (GTK_ENTRY(private->sentry));
+	pat_locale = g_locale_from_utf8 (pattern, -1, NULL, NULL, NULL);
+	index = gtk_combo_box_get_active (GTK_COMBO_BOX(private->scombo));
+	switch (index)
+	{
+		case 0:
+			cur_dir = cxp_dir_view_get_current_directory (CXP_DIR_VIEW(private->dirview));
+			cmd = g_strdup_printf("find '%s' -iname '%s'", cur_dir, pat_locale);
+			g_free (cur_dir);
+			break;
+		case 1:
+			cmd = g_strdup_printf("locate '%s'", pat_locale);
+			break;
+		/*
+		case 2:
+			cmd = g_strdup_printf("namazu --list '%s'", pat_locale);
+			break;
+		*/
+		default:
+			return;
+	}
+	cmd_utf8 = g_locale_to_utf8(cmd, -1, NULL, NULL, NULL);
+	cid = gtk_statusbar_get_context_id (GTK_STATUSBAR(private->statusbar), "SearchContext");
+	gtk_statusbar_push (GTK_STATUSBAR(private->statusbar), cid, cmd_utf8);
+	cxp_right_pane_search_file (CXP_RIGHT_PANE(private->right_pane), cmd);
+	//gtk_statusbar_pop (GTK_STATUSBAR(private->statusbar), cid);
+	g_free (cmd);
+	g_free (cmd_utf8);
+	g_free (pat_locale);
+}
+
+static void cxplorer_window_search_end (GtkWidget *widget, gpointer user_data)
+{
+	CxplorerWindowPrivate *private = CXPLORER_WINDOW_GET_PRIVATE (user_data);
+	gint cid;
+
+	cid = gtk_statusbar_get_context_id (GTK_STATUSBAR(private->statusbar), "SearchContext");
+	gtk_statusbar_pop (GTK_STATUSBAR(private->statusbar), cid);
+}
+
 /**
  * This function is called back when value of TextEntry is changed by user.
  * @param gwCaller [in] GtkWidget which call this function.
